<?php
// $Id: regcode.admin.php,v 1.1.2.4 2009/06/19 01:47:11 aidan Exp $

require_once('regcode.api.php');

/**
 * Return the form associated with the module settings.
 *
 * @return
 *   The settings form.
 */
function regcode_admin_settings() {
  $form = array();
  
  $form['regcode_optional'] = array(
    '#type' => 'checkbox',
    '#title' => t("Make registration code optional"),
    '#default_value' => variable_get('regcode_optional', 0),
    '#description' => t('If checked, users can register without a registration code. ') .
                      t('Alternatively, there is also a permission to allow certain roles (e.g. admins) to bypass´registration code entry for administratively added users.'),
  );
  
  $form['regcode_fieldset_title'] = array(
    '#type' => 'textfield',
    '#title' => t("Field title"),
    '#description' => t('The title of the registration code textfield'),
    '#default_value' => variable_get('regcode_fieldset_title', 'Registration Code'),
  );

  $form['regcode_field_title'] = array(
    '#type' => 'textfield',
    '#title' => t("Field title"),
    '#description' => t('The title of the registration code textfield'),
    '#default_value' => variable_get('regcode_field_title', 'Registration Code'),
  );

  $form['regcode_field_description'] = array(
    '#type' => 'textfield',
    '#title' => t("Field description"),
    '#description' => t('The description of the registration code textfield'),
    '#default_value' => variable_get('regcode_field_description', 'Please enter your registration code.'),
  );

  return system_settings_form($form);
}


/**
 * Create a registration code
 *
 * @return
 *   The settings form.
 */
function regcode_admin_create() {
  $form = array();
  
  $form['regcode_create_code'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Registration code"),
    '#description'   => t('The registration code as the user would enter it. Leave blank to have code generated.'),
  );

  $form['regcode_create_category'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Assign a category"),
    '#size'          => 30,
    '#autocomplete_path' => 'admin/user/regcodes/autocomplete',
    '#description'   => t('Optional classification category for code management'),
  );
  
  $form['regcode_create_maxuses'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Maximum number of uses"),
    '#default_value' => 1,
    '#size'          => 10,
    '#description'   => t('Whether this code can only be used once, or multiple times'),
  );

  $form['regcode_create_begins'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Begins"),
    '#description'   => t('When this code should activate (leave blank to activate immediately)'),
  );
  
  $form['regcode_create_expires'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Expires"),
    '#description'   => t('When this code should expire (leave blank for no expiry)'),
  );

  $form['regcode_create_active'] = array(
    '#type'          => 'checkbox',
    '#title'         => t("Enable code"),
    '#default_value' => 1,
    '#description'   => t('Whether this code can be used immediately'),
  );
  
  $form['regcode_generate'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("Bulk code generation"),
    '#description'   => t('These options are ignored if a registration code is specified'),
  );
  
  $form['regcode_generate']['regcode_generate_number'] = array(
    '#type'          => 'textfield',
    '#default_value' => 1,
    '#size'          => 10,
    '#title'         => t('Number of codes'),
  );
  
  $form['regcode_generate']['regcode_generate_length'] = array(
    '#type'          => 'textfield',
    '#default_value' => variable_get('regcode_generate_length', '10'),
    '#size'          => 10,
    '#title'         => t('Length of codes'),
    '#description'   => t('How many characters should be used in the generated codes'),
  );
  
  $form['regcode_generate']['regcode_generate_format'] = array(
    '#type'          => 'select',
    '#default_value' => variable_get('regcode_generate_format', 'alpha'),
    '#title'         => t('Format of the generated codes'),
    '#options'       => array('alpha' => 'Letters', 'numeric' => 'Numbers', 'alphanum' => 'Numbers & Letters', 'hexadec' => 'Hexadecimal'),
  );
  
  $form['regcode_generate']['regcode_generate_case'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Uppercase generated codes'),
    '#default_value' => variable_get('regcode_generate_case', ''),
  );
  
  $form['regcode_create_submit'] = array(
    '#type'          => 'submit',
    '#value'         => t("Create"),
  );
  
  return $form;
}


/**
 * Validator for create form
 */
function regcode_admin_create_validate($form, &$form_state) {
  if (!empty($form_state['values']['regcode_create_begins']) &&
      strtotime($form_state['values']['regcode_create_begins']) < 1) {

    form_set_error('regcode_create_code', 'Invalid begin date');      
  }
  
  if (!empty($form_state['values']['regcode_create_expires']) &&
      strtotime($form_state['values']['regcode_create_expires']) < 1) {

    form_set_error('regcode_create_code', 'Invalid expiry date');      
  }
}


/**
 * Submit handler for regcode_admin_create
 */
function regcode_admin_create_submit($form, &$form_state) {
  $code = array();
  if (!empty($form_state['values']['regcode_create_begins'])) {
    $code['begins'] = date('Y-m-d h:j:s', strtotime($form_state['values']['regcode_create_begins']));
  }
  
  if (!empty($form_state['values']['regcode_create_expires'])) {
    $code['expires'] = date('Y-m-d h:j:s', strtotime($form_state['values']['regcode_create_expires']));
  }
  
  $code['category']   = $form_state['values']['regcode_create_category'];
  $code['is_active']  = $form_state['values']['regcode_create_active'];
  $code['maxuses']    = $form_state['values']['regcode_create_maxuses'];
  $code['code']       = $form_state['values']['regcode_create_code'];
  
  for ($i = 0; $i < $form_state['values']['regcode_generate_number']; $i++) {
    if ($form_state['values']['regcode_generate_number'] > 1 ||
        empty($form_state['values']['regcode_create_code'])) {
        
      $code['code'] = regcode_generate_code($form_state['values']['regcode_generate_length'],
        $form_state['values']['regcode_generate_format'], 
        $form_state['values']['regcode_generate_case']);
    }
    
    if (regcode_save_code($code, 'skip')) {
      drupal_set_message(t('Created registration code (@code)', array('@code' => $code['code'])));
    } else {
      drupal_set_message(t('Unable to create code (@code) as code already exists', array('@code' => $code['code'])), 'warning');  
    }
  }
}


/**
 * Return the code list page content with(in) the according filter form
 *
 * @return
 *   The settings form.
 */
function regcode_admin_list() {

  $form = array();

  $form['#method'] = 'get';

  $codes = regcode_get_codes();
  $keys = array('ID', 'UID', 'Created', 'Lastused', 'Begins', 'Expires', 'Category', 'Code', 'Active', 'Max uses', 'Uses');
  $output = t('No codes found');
  if (!empty($codes)) {
    $output = theme('table', $keys, $codes);
  }
  
  $form['list'] = array(
      '#type'  => 'markup',
      '#value' => $output,
  );

  return $form;
}


/**
 * Return the form associated with the registration code import admin page
 *
 * @return
 *   The import form array
 */
function regcode_admin_import() {
  require_once 'regcode.install';

  // Build the descriptions
  $action_text = '<p>Action to perform when importing the data.</p>' .
                 '<ul><li><strong>Skip</strong>: Add new codes skipping those which already exists.</li>' .
                 '<li><strong>Overwrite</strong>: Add new codes overwriting those which already exists.</li>' .
                 '<li><strong>Clean</strong>: Erases *all* existing codes before importing new codes.</li></ul>';

  $fieldorder_text = '<p>Comma separated list mapping regcode fields to CSV fields, e.g. "IGNORE, code, rid, info".</p>' .
                     '<p>If you leave this field blank, the first CSV row will be used.</p>' .
                     '<ul>';
  $schema = regcode_schema('regcode');
  foreach ($schema['regcode']['fields'] as $field => $value) {
    $fieldorder_text .= sprintf('<li><strong>%s</strong>: %s</li>', $field, $value['description']);
  }
  $fieldorder_text .= '</ul>';
                             
  // Build the form                         
  $form['#attributes']['enctype'] = 'multipart/form-data';

  $form['regcode_import_file'] = array(
      '#type' => 'file',
      '#title' => t("File"),
      '#description' => t('File upload with data to add/import (plaintext list or CSV format)'),
  );

  $form['regcode_import_action'] = array(
      '#type' => 'select',
      '#title' => t("Action"),
      '#default_value' => variable_get('regcode_import_action', 'skip'),
      '#options' => array('skip'      => t('Skip'),
                          'overwrite' => t('Overwrite'),
                          'clean'     => t('Clean')),
      '#description' => t($action_text),
  );

  $form['regcode_import_delimiter'] = array(
    '#type'          => 'textfield',
    '#size'          => 5,
    '#title'         => t("Field delimiter"),
    '#description'   => t('Character used as delimiter in csv import of registration codes'),
    '#default_value' => variable_get('regcode_import_delimiter', ','),
  );

  $form['regcode_import_enclosure'] = array(
    '#type'          => 'textfield',
    '#size'          => 5,
    '#title'         => t("Text enclosure"),
    '#default_value' => variable_get('regcode_import_enclosure', '"'),
    '#description'   => t('Character used as enclosure around text content fields on CSV import.'),
  );
  
  $form['regcode_import_fieldorder'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Field order"),
    '#default_value' => variable_get('regcode_import_fieldorder', 'code'),
    '#description'   => t($fieldorder_text),
  );

  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => 'Import',
  );

  return $form;
}


/**
 * Handle the processing of a submitted import form
 *
 */
function regcode_admin_import_validate($form, &$form_state) {
  variable_set('regcode_import_delimiter',  $form_state['values']['regcode_import_delimiter']);
  variable_set('regcode_import_enclosure',  $form_state['values']['regcode_import_enclosure']);
  variable_set('regcode_import_fieldorder', $form_state['values']['regcode_import_fieldorder']);
  
  // Validators for file
  $validators = array(
    'file_validate_extensions' => array('csv', 'txt'),
  );

  // Save the file
  if ($file = file_save_upload('regcode_import_file', $validators, file_directory_path())) {
    $form_state['values']['fileobj'] = $file;
    drupal_set_message('File successfully saved');
  } else {
    form_set_error('regcode_import_file', 'Unable to parse CSV file');
    return false;
  }
}


/**
 * Handle the processing of a submitted import form
 *
 */
function regcode_admin_import_submit($form, &$form_state) {
  
  // Set some variblaes
  $action    = $form_state['values']['regcode_import_action'];
  $delimiter = $form_state['values']['regcode_import_delimiter'];
  $enclosure = $form_state['values']['regcode_import_enclosure'];
  $fields    = explode(',', $form_state['values']['regcode_import_fieldorder']);
  $fields    = array_map('trim', $fields);
  $rows = $success = 0;
  
  // Parse and save
  $fh        = fopen($form_state['values']['fileobj']->destination, 'r');
  while ($row = fgetcsv($fh, 1000, $delimiter, $enclosure)) {
    $rows++;
    $row  = array_slice($row, 0, count($fields));
    $code = array_combine($fields, $row);
    
    // Check dates
    if (!empty($code['begins'])) {
      $code['begins'] = strtotime($code['begins']);
      if ($code['begins'] < 1) continue;
    }
    if (!empty($code['expires'])) {
      $code['expires'] = strtotime($code['expires']);
      if ($code['expires'] < 1) continue;
    }

    // Defaults
    if (empty($code['is_active'])) $code['is_active'] = 1;
    if (empty($code['maxuses'])) $code['maxuses'] = 1;
  
    $status = regcode_save_code($code, $action);
    if ($status) $success++;
  }
  
  // Status
  drupal_set_message(t('Successfully inserted @rows rows out of @total', array('@rows' => $success, '@total' => $rows)));
  
  // Close
  fclose($fh);
}



/**
 * Autocomplete helper for categories
 */
function regcode_admin_categories($string) {
  $matches = array();
  if (!empty($string)) {
    $result = db_query_range("SELECT DISTINCT category FROM {regcode} WHERE LOWER(category) LIKE LOWER('%s%%')", $string, 0, 10);
    while ($regcode = db_fetch_object($result)) {
      $matches[$regcode->category] = check_plain($regcode->category);
    }
  }
  
  print drupal_to_js($matches);
}


/**
 *
 */
function regcode_admin_export() {
  return 'not implemented';
}
