<?php
// $Id: regcode.api.php,v 1.1.2.3 2009/06/19 01:47:11 aidan Exp $

/**
 * Return text message requested by given identifier/constant
 *
 * @param $code
 *  The message code
 * @return
 *  The text of the message.
 */
function regcode_message($code) {
  $messages[REGCODE_VALIDITY_NOTEXISTING]  = 'Registration code does not exist.';
  $messages[REGCODE_VALIDITY_NOTAVAILABLE] = 'Registration code is not available yet.';
  $messages[REGCODE_VALIDITY_TAKEN]        = 'Registration code has already been used.';
  $messages[REGCODE_VALIDITY_EXPIRED]      = 'Registration code has expired.';
  return $messages[$code];
}


/**
 * Retrieve a particular registration code identified by given code id,
 * or assign it to a given user
 *
 * @param $code
 *  The code identifiert string
 * @param $assign_to_uid
 *  Optional user id to assign the given code to
 * @return
 *  The data array of the code to retrieve, or a boolean result (TRUE=successfully assigned to given uid) if assign_to_uid has been given.
 */
function regcode_get_code($code, $use_code = FALSE) {

  // look up and retrieve code record from DB
  $code = db_fetch_array(db_query("SELECT code, begins, expires, is_active, maxuses, uses FROM {regcode} WHERE code = '%s'", $code));

  // check validity
  if (!is_array($code)) {
    return REGCODE_VALIDITY_NOTEXISTING;
  }
  
  if (!$code['is_active']) {
    return REGCODE_VALIDITY_NOTAVAILABLE;
  }
  
  if (!empty($code['begins']) && strtotime($code['begins']) > time()) {
    return REGCODE_VALIDITY_NOTAVAILABLE;
  }
  
  if (!empty($code['expires']) && strtotime($code['expires']) < time()) {
    return REGCODE_VALIDITY_EXPIRED;
  }
  
  if ($code['uses'] >= $code['maxuses']) {
    return REGCODE_VALIDITY_TAKEN;
  }
  
  // optionally mark this code as taken for the given user uid
  if ($use_code) {
    watchdog('regcode', 'The registration code (@code) was used by UID #uid', array('@code' => $code['code'], '#uid' => $use_code));
    db_query("UPDATE {regcode} SET uses=uses+1, lastused=NOW(), uid=%d WHERE code='%s'", $use_code, $code['code']);
  }

  return $code;
}


/**
 * Return the database query result for the given options to list codes
 *
 * @return
 *   An array list of code data arrays
 */
function regcode_get_codes($options = array()) {

  $result = db_query('SELECT rid, uid, created, lastused, begins, expires, category, code, is_active, maxuses, uses FROM {regcode}');
  
  $data = array();
  while ($row = db_fetch_array($result)) {
    $data[] = $row;
  }
  
  return $data;
}


/**
 * Save given code array to a record in the db
 *
 * @param $code
 *   An array of code data fields (dates must be timestamps)
 * @param $action
 *   Action to perform when saving the code:
 *     overwrite  -  overwrite any existing code with same code identifier
 *     skip       -  skip and do not save code if there is any existing code with same code identifier
 * @return
 *   An array list of code data arrays
 */
function regcode_save_code($code, $action = 'overwrite') {
  // Sanity check
  if (empty($code) || empty($code['code'])) {
    return FALSE;
  }
  
  if ($action == 'overwrite') {
    db_query('DELETE FROM {regcode} WHERE code = "%s" LIMIT 1', $code['code']);
  }
  
  // Format data
  $format   = 'Y-m-d h:j:s';
  $begins   = empty($code['begins'])   ? 'NULL' : '"' . date($format, $code['begins']) . '"';
  $expires  = empty($code['expires'])  ? 'NULL' : '"' . date($format, $code['expires']) . '"';
  $category = empty($code['category']) ? 'NULL' : '"' . check_plain($code['category']) . '"';

  // Insert
  $sql = "INSERT INTO {regcode} (created,begins,expires,category,code,is_active,maxuses)
          VALUES (NOW(), $begins, $expires, $category, '%s', %d, %d)";
  $result = db_query($sql,
    $code['code'],
    $code['is_active'],
    $code['maxuses']);
    
  return $result;
}


/**
 * Clean up codes
 *
 * @param $op
 *   Action to perform when cleaning the code:
 *     truncate      -  truncate all codes from db
 *     expired       -  remove expired codes
 *     used          -  remove one-time codes that have been used
 */
function regcode_clean_codes($op = 'truncate') {
  switch ($op) {
    case 'truncate':
      db_query('TRUNCATE {regcode}');
      break;
    
    case 'expired':
      db_query('DELETE FROM {regcode} WHERE expire < NOW()');
      break;
    
    case 'used':
      db_query('DELETE FROM {regcode} WHERE uses >= maxuses');
      break;
  }
}


/**
 * Generate a code
 */
function regcode_generate_code($length, $output, $case) {
  static $seeded = false;
  
  // Possible seeds
  $outputs['alpha']    = 'abcdefghijklmnopqrstuvwqyz';
  $outputs['numeric']  = '0123456789';
  $outputs['alphanum'] = 'abcdefghijklmnopqrstuvwqyz0123456789';
  $outputs['hexadec']  = '0123456789abcdef';

  // Choose seed
  if (isset($outputs[$output])) {
    $output = $outputs[$output];
  }

  // Seed generator (only do this once per invocation)
  if (!$seeded) {
    list($usec, $sec) = explode(' ', microtime());
    $seed = (float) $sec + ((float) $usec * 100000);
    mt_srand($seed);
    $seeded = true;
  }
  
  // Generate
  $str = '';
  $output_count = strlen($output);
  for ($i = 0; $length > $i; $i++) {
    $str .= $output{mt_rand(0, $output_count - 1)};
  }

  if ($case) {
    $str = strtoupper($str);
  }
  
  return $str;
}